unit MainUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  HuffUnit;

type

  { TMainForm }

  TMainForm = class(TForm)
    ButtonUlozTxt: TButton;
    ButtonOtvorTxt: TButton;
    ButtonVymazat: TButton;
    ButtonKoniec: TButton;
    ButtonOtvor: TButton;
    ButtonUloz: TButton;
    ButtonZakoduj: TButton;
    CheckBoxTrvanie: TCheckBox;
    LabelZakodovaneV: TLabel;
    LabelTextV: TLabel;
    LabelKody: TLabel;
    LabelZakodovane: TLabel;
    LabelText: TLabel;
    MemoText: TMemo;
    MemoZakodovane: TMemo;
    MemoKody: TMemo;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    procedure ButtonKoniecClick(Sender: TObject);
    procedure ButtonOtvorClick(Sender: TObject);
    procedure ButtonOtvorTxtClick(Sender: TObject);
    procedure ButtonUlozClick(Sender: TObject);
    procedure ButtonUlozTxtClick(Sender: TObject);
    procedure ButtonVymazatClick(Sender: TObject);
    procedure ButtonZakodujClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MemoTextChange(Sender: TObject);
    procedure MemoZakodovaneChange(Sender: TObject);

  private
    { private declarations }
  public
    { public declarations }
  end; 

var
  MainForm: TMainForm;

  Kodovanie: THuffK;

implementation

{$R *.lfm}

{ TMainForm }

procedure TMainForm.ButtonZakodujClick(Sender: TObject);
var
  T1, T2: TDateTime;
begin
  if MemoText.Text <> '' then
  begin
    T1 := Now;
    Kodovanie := Huffman(MemoText.Text);
    T2 := Now;
    if CheckBoxTrvanie.Checked then
      ShowMessage(FormatDateTime('"Huffmanovho kódovanie OK. Trvanie: " hh:nn:ss.zzz', T2 - T1));

    T1 := Now;
    MemoZakodovane.Text := Kodovanie.Text;
    MemoKody.Text := KodyText(Kodovanie.Kody);
    T2 := Now;
    if CheckBoxTrvanie.Checked then
      ShowMessage(FormatDateTime('"Výpis OK. Trvanie: " hh:nn:ss.zzz', T2 - T1));
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Kodovanie.Text := '';
end;

procedure TMainForm.MemoTextChange(Sender: TObject);
begin
  LabelTextV.Caption:= 'Velkosť: ' + IntToStr(Length(MemoText.Text)) + ' bajtov';
end;

procedure TMainForm.MemoZakodovaneChange(Sender: TObject);
begin
  LabelZakodovaneV.Caption:= 'Velkosť: ' + IntToStr(Length(MemoZakodovane.Text) div 8) + ' bajtov';
end;

procedure TMainForm.ButtonKoniecClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.ButtonOtvorClick(Sender: TObject);
var
  NazovSuboru: string;
begin
  OpenDialog.InitialDir := Application.Location;
  OpenDialog.Filter := 'Binárne súbory (*.dat)|*.dat';


  OpenDialog.Title := 'Otvoriť súbor so zakódovaným textom';

  if OpenDialog.Execute then
  begin
    if AnsiPos('.dat', OpenDialog.FileName) = 0 then
      NazovSuboru := OpenDialog.FileName + '.dat'
    else
      NazovSuboru := OpenDialog.FileName;

    if not FileExists(NazovSuboru) then
    begin
      MessageDlg('Huffmanovo kódovanie', 'Súbor, ktorý chcete otvoriť neexistuje!', mtInformation, [mbOk], 0);
      Exit;
    end;

    Kodovanie := Otvor(NazovSuboru);

    MemoZakodovane.Text := Kodovanie.Text;
    MemoKody.Text := KodyText(Kodovanie.Kody);
    MemoText.Text := DekodujHuffmana(Kodovanie.Text, Kodovanie.Kody);
  end;
end;

procedure TMainForm.ButtonOtvorTxtClick(Sender: TObject);
var
  NazovSuboru: string;
begin
  OpenDialog.InitialDir := Application.Location;
  OpenDialog.Filter := 'Všetky súbory (*.txt)|*.txt';


  OpenDialog.Title := 'Otvoriť textový súbor';

  if OpenDialog.Execute then
  begin
    if AnsiPos('.txt', OpenDialog.FileName) = 0 then
      NazovSuboru := OpenDialog.FileName + '.txt'
    else
      NazovSuboru := OpenDialog.FileName;

    if not FileExists(NazovSuboru) then
    begin
      MessageDlg('Huffmanovo kódovanie', 'Súbor, ktorý chcete otvoriť neexistuje!', mtInformation, [mbOk], 0);
      Exit;
    end;

    MemoText.Lines.LoadFromFile(NazovSuboru);
  end;
end;

procedure TMainForm.ButtonUlozClick(Sender: TObject);
var
  NazovSuboru: string;
begin
  if Kodovanie.Text = '' then
    Exit;

  SaveDialog.InitialDir := Application.Location;
  SaveDialog.Filter := 'Binárne súbory (*.dat)|*.dat';


  SaveDialog.Title := 'Uložiť súbor so zakódovaným textom';

  if SaveDialog.Execute then
  begin
    if AnsiPos('.dat', SaveDialog.FileName) = 0 then
      NazovSuboru := SaveDialog.FileName + '.dat'
    else
      NazovSuboru := SaveDialog.FileName;

    if FileExists(NazovSuboru) then
      if MessageDlg('Huffmanovo kódovanie', 'Súbor so zadaným názvom už existuje. Prajete si ho nahradiť?', mtInformation, [mbYes, mbNo], 0) <> mrYes then
        Exit;

    Uloz(Kodovanie.Text, Kodovanie.Kody, NazovSuboru);
  end;
end;

procedure TMainForm.ButtonUlozTxtClick(Sender: TObject);
var
  NazovSuboru: string;
begin
  SaveDialog.InitialDir := Application.Location;
  SaveDialog.Filter := 'Všetky súbory (*.txt)|*.txt';


  SaveDialog.Title := 'Uložiť textový súbor';

  if SaveDialog.Execute then
  begin
    if AnsiPos('.txt', SaveDialog.FileName) = 0 then
      NazovSuboru := SaveDialog.FileName + '.txt'
    else
      NazovSuboru := SaveDialog.FileName;

    if FileExists(NazovSuboru) then
      if MessageDlg('Huffmanovo kódovanie', 'Súbor so zadaným názvom už existuje. Prajete si ho nahradiť?', mtInformation, [mbYes, mbNo], 0) <> mrYes then
        Exit;

    MemoText.Lines.SaveToFile(NazovSuboru);
  end;
end;

procedure TMainForm.ButtonVymazatClick(Sender: TObject);
begin
  Kodovanie.Text := '';

  MemoText.Text := '';
  MemoZakodovane.Text := '';
  MemoKody.Text := '';
end;

end.

